@@ -1,5 +1,4 @@
 import sys
-import argparse
 import asyncio
 import statistics
 import time
@@ -10,7 +9,6 @@ import tempfile
 import traceback
 from typing import Optional, Union
 import json
-import subprocess
 from pathlib import Path
 
 from fastapi import FastAPI, HTTPException, status, Request
@@ -48,29 +46,40 @@ from openai.types.responses import (
 )
 
 import lemonade.api as lemonade_api
-from lemonade_server.model_manager import ModelManager
-from lemonade_server.pydantic_models import (
-    DEFAULT_MAX_NEW_TOKENS,
-    LoadConfig,
-    CompletionRequest,
-    ChatCompletionRequest,
-    ResponsesRequest,
-    PullConfig,
-    DeleteConfig,
-)
-from lemonade.tools.management_tools import ManagementTool
-import lemonade.tools.server.llamacpp as llamacpp
+from lemonade.tools.server.wrapped_server import WrappedServer
+from lemonade.tools.server.llamacpp import LlamaServer
 from lemonade.tools.server.tool_calls import extract_tool_calls, get_tool_call_pattern
 from lemonade.tools.server.webapp import get_webapp_html
 from lemonade.tools.server.utils.port import lifespan
 
+from lemonade_server.model_manager import ModelManager
+from lemonade_server.pydantic_models import (
+    DEFAULT_PORT,
+    DEFAULT_HOST,
+    DEFAULT_LOG_LEVEL,
+    DEFAULT_LLAMACPP_BACKEND,
+    DEFAULT_CTX_SIZE,
+    LoadConfig,
+    CompletionRequest,
+    ChatCompletionRequest,
+    EmbeddingsRequest,
+    RerankingRequest,
+    ResponsesRequest,
+    PullConfig,
+    DeleteConfig,
+    LogLevelConfig,
+)
+from lemonade_server.settings import save_setting
+
+# Set to a high number to allow for interesting experiences in real apps
+# Tests should use the max_new_tokens argument to set a lower value
+DEFAULT_MAX_NEW_TOKENS = 1500
+
 # Only import tray on Windows
 if platform.system() == "Windows":
+    # pylint: disable=ungrouped-imports
     from lemonade.tools.server.tray import LemonadeTray, OutputDuplicator
 
-DEFAULT_PORT = 8000
-DEFAULT_LOG_LEVEL = "info"
-
 
 class ServerModel(Model):
     """
@@ -124,7 +133,7 @@ class StopOnEvent:
         return self.stop_event.is_set()
 
 
-class Server(ManagementTool):
+class Server:
     """
     Open a web server that apps can use to communicate with the LLM.
 
@@ -142,11 +151,27 @@ class Server(ManagementTool):
     - /api/v1/models: list all available models.
     """
 
-    unique_name = "serve"
-
-    def __init__(self):
+    def __init__(
+        self,
+        port: int = DEFAULT_PORT,
+        host: str = DEFAULT_HOST,
+        log_level: str = DEFAULT_LOG_LEVEL,
+        ctx_size: int = DEFAULT_CTX_SIZE,
+        tray: bool = False,
+        log_file: str = None,
+        llamacpp_backend: str = DEFAULT_LLAMACPP_BACKEND,
+    ):
         super().__init__()
 
+        # Save args as members
+        self.port = port
+        self.host = host
+        self.log_level = log_level
+        self.ctx_size = ctx_size
+        self.tray = tray
+        self.log_file = log_file
+        self.llamacpp_backend = llamacpp_backend
+
         # Initialize FastAPI app
         self.app = FastAPI(lifespan=lifespan)
 
@@ -184,9 +209,6 @@ class Server(ManagementTool):
         self.output_tokens = None
         self.decode_token_times = None
 
-        # Input truncation settings
-        self.truncate_inputs = False
-
         # Store debug logging state
         self.debug_logging_enabled = logging.getLogger().isEnabledFor(logging.DEBUG)
 
@@ -210,11 +232,8 @@ class Server(ManagementTool):
         # Add lock for load/unload operations
         self._load_lock = asyncio.Lock()
 
-        # Subprocess handle for llama_server.exe
-        self.llama_server_process: subprocess.Popen = None
-
-        # Telemetry instance for llama server
-        self.llama_telemetry = llamacpp.LlamaTelemetry()
+        # Subprocess handle for wrapped instance of llama_server.exe, etc.
+        self.wrapped_server: WrappedServer = None
 
     def setup_routes(self, api_prefixes: list[str]):
         for prefix in api_prefixes:
@@ -226,73 +245,105 @@ class Server(ManagementTool):
             self.app.get(f"{prefix}/health")(self.health)
             self.app.get(f"{prefix}/halt")(self.halt_generation)
             self.app.get(f"{prefix}/stats")(self.send_stats)
+            self.app.get(f"{prefix}/system-info")(self.get_system_info)
             self.app.post(f"{prefix}/completions")(self.completions)
             self.app.post(f"{prefix}/responses")(self.responses)
+            self.app.post(f"{prefix}/log-level")(self.set_log_level)
 
             # OpenAI-compatible routes
             self.app.post(f"{prefix}/chat/completions")(self.chat_completions)
+            self.app.post(f"{prefix}/embeddings")(self.embeddings)
             self.app.get(f"{prefix}/models")(self.models)
 
-    @staticmethod
-    def parser(add_help: bool = True) -> argparse.ArgumentParser:
-        parser = __class__.helpful_parser(
-            short_description="Launch an industry-standard LLM server",
-            add_help=add_help,
-        )
+            # JinaAI routes (jina.ai/reranker/)
+            self.app.post(f"{prefix}/reranking")(self.reranking)
+            self.app.post(f"{prefix}/rerank")(self.reranking)
 
-        # Only add the tray option on Windows
-        if platform.system() == "Windows":
-            parser.add_argument(
-                "--tray",
-                action="store_true",
-                help="Run the server in system tray mode",
+    async def set_log_level(self, config: LogLevelConfig):
+        """
+        Set the logging level of the server.
+        """
+        try:
+            log_level_upper = config.level.upper()
+            numeric_level = getattr(logging, log_level_upper, None)
+            if not isinstance(numeric_level, int):
+                raise ValueError(f"Invalid log level: {config.level}")
+
+            # Get the root logger
+            logger = logging.getLogger()
+            logger.setLevel(numeric_level)
+
+            # Update all handlers
+            for handler in logger.handlers:
+                handler.setLevel(numeric_level)
+
+            logging.getLogger("uvicorn.error").setLevel(numeric_level)
+            self.debug_logging_enabled = numeric_level <= logging.DEBUG
+
+            # Save the setting
+            save_setting("log_level", config.level)
+
+            logging.info(f"Log level changed to: {config.level}")
+            return {"status": "success", "message": f"Log level set to {config.level}"}
+        except Exception as e:
+            raise HTTPException(
+                status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
+                detail=f"Failed to set log level: {str(e)}",
             )
 
-        parser.add_argument(
-            "--port",
-            required=False,
-            type=int,
-            default=DEFAULT_PORT,
-            help=f"Port number to run the server on (default: {DEFAULT_PORT})",
-        )
-        parser.add_argument(
-            "--log-level",
-            required=False,
-            type=str,
-            default=DEFAULT_LOG_LEVEL,
-            choices=["critical", "error", "warning", "info", "debug", "trace"],
-            help=f"Logging level (default: {DEFAULT_LOG_LEVEL})",
-        )
+    def _log_request_parameters(self, request, endpoint_name: str):
+        """
+        Log request parameters excluding content fields like messages, prompt, or input.
 
-        parser.add_argument(
-            "--log-file",
-            required=False,
-            type=str,
-            help="Path to the log file",
-        )
+        Args:
+            request: Any request object (CompletionRequest, ChatCompletionRequest, etc.)
+            endpoint_name: Name of the endpoint for logging context
+        """
+        if not logging.getLogger().isEnabledFor(logging.DEBUG):
+            return
 
-        return parser
+        # Fields to exclude from logging (content fields)
+        excluded_fields = {"messages", "prompt", "input"}
+
+        # Get all attributes from the request object
+        request_params = {}
+        if hasattr(request, "__dict__"):
+            # For pydantic models, get the dict representation
+            if hasattr(request, "model_dump"):
+                all_params = request.model_dump()
+            elif hasattr(request, "dict"):
+                all_params = request.dict()
+            else:
+                all_params = request.__dict__
+
+            # Filter out excluded fields and add special handling for certain fields
+            for key, value in all_params.items():
+                if key not in excluded_fields:
+                    # Special handling for tools field - show count instead of full content
+                    if key == "tools" and value is not None:
+                        request_params[key] = (
+                            f"{len(value)} tools" if isinstance(value, list) else value
+                        )
+                    # Special handling for input type in responses
+                    elif key == "input" and hasattr(request, "input"):
+                        request_params["input_type"] = type(value).__name__
+                    else:
+                        request_params[key] = value
+
+        logging.debug(f"{endpoint_name} request parameters: {request_params}")
 
     def _setup_server_common(
         self,
-        port: int,
-        truncate_inputs: bool = False,
-        log_level: str = DEFAULT_LOG_LEVEL,
         tray: bool = False,
-        log_file: str = None,
         threaded_mode: bool = False,
     ):
         """
         Common setup logic shared between run() and run_in_thread().
 
         Args:
-            port: Port number for the server
-            truncate_inputs: Whether to truncate inputs if they exceed max length
-            log_level: Logging level to configure
+            tray: Whether to run the server in tray mode
             threaded_mode: Whether this is being set up for threaded execution
         """
-        # Store truncation settings
-        self.truncate_inputs = truncate_inputs
 
         # Define TRACE level
         logging.TRACE = 9  # Lower than DEBUG which is 10
@@ -310,18 +361,20 @@ class Server(ManagementTool):
             logging.getLogger("uvicorn.error").setLevel(logging.WARNING)
         else:
             # Configure logging to match uvicorn's format
-            logging_level = getattr(logging, log_level.upper())
+            logging_level = getattr(logging, self.log_level.upper())
 
             # Set up file handler for logging to lemonade.log
             uvicorn_formatter = uvicorn.logging.DefaultFormatter(
                 fmt="%(levelprefix)s %(message)s",
                 use_colors=True,
             )
-            if not log_file:
-                log_file = tempfile.NamedTemporaryFile(
+            if not self.log_file:
+                self.log_file = tempfile.NamedTemporaryFile(
                     prefix="lemonade_", suffix=".log", delete=False
                 ).name
-            file_handler = logging.FileHandler(log_file, mode="a", encoding="utf-8")
+            file_handler = logging.FileHandler(
+                self.log_file, mode="a", encoding="utf-8"
+            )
             file_handler.setLevel(logging_level)
             file_handler.setFormatter(uvicorn_formatter)
 
@@ -341,12 +394,14 @@ class Server(ManagementTool):
         self.debug_logging_enabled = logging.getLogger().isEnabledFor(logging.DEBUG)
         if tray:
             # Save original stdout/stderr
-            sys.stdout = OutputDuplicator(log_file, sys.stdout)
-            sys.stderr = OutputDuplicator(log_file, sys.stderr)
+            sys.stdout = OutputDuplicator(self.log_file, sys.stdout)
+            sys.stderr = OutputDuplicator(self.log_file, sys.stderr)
 
             # Open lemonade server in tray mode
             # lambda function used for deferred instantiation and thread safety
-            LemonadeTray(log_file, port, lambda: Server()).run()
+            LemonadeTray(
+                self.log_file, self.port, lambda: self, log_level=self.log_level
+            ).run()
             sys.exit(0)
 
         if self.debug_logging_enabled:
@@ -355,47 +410,27 @@ class Server(ManagementTool):
 
         # Let the app know what port it's running on, so
         # that the lifespan can access it
-        self.app.port = port
+        self.app.port = self.port
+        # FastAPI already has a `host` function and we cannot use `_host` as
+        # PyLint will believe its private
+        self.app.host_ = self.host
 
-    def run(
-        self,
-        # ManagementTool has a required cache_dir arg, but
-        # we always use the default cache directory
-        _=None,
-        port: int = DEFAULT_PORT,
-        log_level: str = DEFAULT_LOG_LEVEL,
-        truncate_inputs: bool = False,
-        tray: bool = False,
-        log_file: str = None,
-    ):
+    def run(self):
         # Common setup
         self._setup_server_common(
-            port=port,
-            truncate_inputs=truncate_inputs,
-            log_level=log_level,
             threaded_mode=False,
-            tray=tray,
-            log_file=log_file,
+            tray=self.tray,
         )
 
-        uvicorn.run(self.app, host="localhost", port=port, log_level=log_level)
+        uvicorn.run(self.app, host=self.host, port=self.port, log_level=self.log_level)
 
-    def run_in_thread(
-        self,
-        port: int = DEFAULT_PORT,
-        host: str = "localhost",
-        log_level: str = "warning",
-        truncate_inputs: bool = False,
-    ):
+    def run_in_thread(self, host: str = "localhost"):
         """
         Set up the server for running in a thread.
         Returns a uvicorn server instance that can be controlled externally.
         """
         # Common setup
         self._setup_server_common(
-            port=port,
-            truncate_inputs=truncate_inputs,
-            log_level=log_level,
             threaded_mode=True,
             tray=False,
         )
@@ -410,8 +445,8 @@ class Server(ManagementTool):
         config = Config(
             app=self.app,
             host=host,
-            port=port,
-            log_level=log_level,
+            port=self.port,
+            log_level=self.log_level,
             log_config=None,
         )
 
@@ -476,9 +511,15 @@ class Server(ManagementTool):
 
         lc = self.initialize_load_config(completion_request)
 
+        # Log request parameters (excluding message content for brevity)
+        self._log_request_parameters(completion_request, "Completions")
+
         # Load the model if it's different from the currently loaded one
         await self.load_llm(lc)
 
+        if self.llm_loaded.recipe == "llamacpp":
+            return self.wrapped_server.completion(completion_request)
+
         # Check if the model supports reasoning
         reasoning_first_token = self.llm_loaded.reasoning
 
@@ -494,6 +535,9 @@ class Server(ManagementTool):
             "message": text,
             "stop": completion_request.stop,
             "temperature": completion_request.temperature,
+            "repeat_penalty": completion_request.repeat_penalty,
+            "top_k": completion_request.top_k,
+            "top_p": completion_request.top_p,
             "max_new_tokens": completion_request.max_tokens,
         }
 
@@ -602,13 +646,14 @@ class Server(ManagementTool):
 
         lc = self.initialize_load_config(chat_completion_request)
 
+        # Log request parameters (excluding message history for brevity)
+        self._log_request_parameters(chat_completion_request, "Chat completions")
+
         # Load the model if it's different from the currently loaded one
         await self.load_llm(lc)
 
         if self.llm_loaded.recipe == "llamacpp":
-            return llamacpp.chat_completion(
-                chat_completion_request, self.llama_telemetry
-            )
+            return self.wrapped_server.chat_completion(chat_completion_request)
 
         # Convert chat messages to text using the model's chat template
         text = self.apply_chat_template(
@@ -646,6 +691,9 @@ class Server(ManagementTool):
             "message": text,
             "stop": chat_completion_request.stop,
             "temperature": chat_completion_request.temperature,
+            "repeat_penalty": chat_completion_request.repeat_penalty,
+            "top_k": chat_completion_request.top_k,
+            "top_p": chat_completion_request.top_p,
             "max_new_tokens": max_new_tokens,
         }
 
@@ -796,6 +844,72 @@ class Server(ManagementTool):
                 created=int(time.time()),
             )
 
+    async def embeddings(self, embeddings_request: EmbeddingsRequest):
+        """
+        Generate embeddings for the provided input.
+        """
+        # Initialize load config from embeddings request
+        lc = LoadConfig(model_name=embeddings_request.model)
+
+        # Load the model if it's different from the currently loaded one
+        await self.load_llm(lc)
+
+        if self.llm_loaded.recipe == "llamacpp":
+            try:
+                return self.wrapped_server.embeddings(embeddings_request)
+            except Exception as e:  # pylint: disable=broad-exception-caught
+                # Check if model has embeddings label
+                model_info = ModelManager().supported_models.get(
+                    self.llm_loaded.model_name, {}
+                )
+                if "embeddings" not in model_info.get("labels", []):
+                    raise HTTPException(
+                        status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
+                        detail="You tried to generate embeddings for a model that is "
+                        "not labeled as an embeddings model. Please use another model "
+                        "or re-register the current model with the 'embeddings' label.",
+                    ) from e
+                else:
+                    raise e
+        else:
+            raise HTTPException(
+                status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
+                detail=f"Embeddings not supported for recipe: {self.llm_loaded.recipe}",
+            )
+
+    async def reranking(self, reranking_request: RerankingRequest):
+        """
+        Rerank documents based on their relevance to a query.
+        """
+        # Initialize load config from reranking request
+        lc = LoadConfig(model_name=reranking_request.model)
+
+        # Load the model if it's different from the currently loaded one
+        await self.load_llm(lc)
+
+        if self.llm_loaded.recipe == "llamacpp":
+            try:
+                return self.wrapped_server.reranking(reranking_request)
+            except Exception as e:  # pylint: disable=broad-exception-caught
+                # Check if model has reranking label
+                model_info = ModelManager().supported_models.get(
+                    self.llm_loaded.model_name, {}
+                )
+                if "reranking" not in model_info.get("labels", []):
+                    raise HTTPException(
+                        status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
+                        detail="You tried to use reranking for a model that is "
+                        "not labeled as a reranking model. Please use another model "
+                        "or re-register the current model with the 'reranking' label.",
+                    ) from e
+                else:
+                    raise e
+        else:
+            raise HTTPException(
+                status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
+                detail=f"Reranking not supported for recipe: {self.llm_loaded.recipe}",
+            )
+
     def apply_chat_template(
         self, messages: list[dict], tools: list[dict] | None = None
     ):
@@ -828,6 +942,9 @@ class Server(ManagementTool):
 
         lc = self.initialize_load_config(responses_request)
 
+        # Log request parameters (excluding message history for brevity)
+        self._log_request_parameters(responses_request, "Responses")
+
         # Load the model if it's different from the currently loaded one
         await self.load_llm(lc)
 
@@ -849,6 +966,9 @@ class Server(ManagementTool):
         generation_args = {
             "message": text,
             "temperature": responses_request.temperature,
+            "repeat_penalty": responses_request.repeat_penalty,
+            "top_k": responses_request.top_k,
+            "top_p": responses_request.top_p,
             "max_new_tokens": responses_request.max_output_tokens,
         }
 
@@ -978,6 +1098,9 @@ class Server(ManagementTool):
         stop: list[str] | str | None = None,
         max_new_tokens: int | None = None,
         temperature: float | None = None,
+        repeat_penalty: float | None = None,
+        top_k: int | None = None,
+        top_p: float | None = None,
     ):
         """
         Core streaming completion logic, separated from response handling.
@@ -1022,29 +1145,21 @@ class Server(ManagementTool):
             )
             self.input_tokens = len(input_ids[0])
 
-        if (
-            self.llm_loaded.max_prompt_length
-            and self.input_tokens > self.llm_loaded.max_prompt_length
-        ):
-            if self.truncate_inputs:
-                # Truncate input ids
-                truncate_amount = self.input_tokens - self.llm_loaded.max_prompt_length
-                input_ids = input_ids[: self.llm_loaded.max_prompt_length]
+        # For non-llamacpp recipes, truncate inputs to ctx_size if needed
+        if self.llm_loaded.recipe != "llamacpp" and self.input_tokens > self.ctx_size:
+            # Truncate input ids
+            truncate_amount = self.input_tokens - self.ctx_size
+            input_ids = input_ids[: self.ctx_size]
 
-                # Update token count
-                self.input_tokens = len(input_ids)
+            # Update token count
+            self.input_tokens = len(input_ids)
 
-                # Show warning message
-                truncation_message = (
-                    f"Input exceeded {self.llm_loaded.max_prompt_length} tokens. "
-                    f"Truncated {truncate_amount} tokens."
-                )
-                logging.warning(truncation_message)
-            else:
-                raise RuntimeError(
-                    f"Prompt tokens ({self.input_tokens}) cannot be greater "
-                    f"than the model's max prompt length ({self.llm_loaded.max_prompt_length})"
-                )
+            # Show warning message
+            truncation_message = (
+                f"Input exceeded {self.ctx_size} tokens. "
+                f"Truncated {truncate_amount} tokens from the beginning."
+            )
+            logging.warning(truncation_message)
 
         # Log the input tokens early to avoid this not showing due to potential crashes
         logging.debug(f"Input Tokens: {self.input_tokens}")
@@ -1068,6 +1183,9 @@ class Server(ManagementTool):
             "pad_token_id": tokenizer.eos_token_id,
             "stopping_criteria": stopping_criteria,
             "temperature": temperature,
+            "repeat_penalty": repeat_penalty,
+            "top_k": top_k,
+            "top_p": top_p,
         }
 
         # Initialize performance variables
@@ -1164,7 +1282,7 @@ class Server(ManagementTool):
         """
         # If using llama server, get telemetry from the telemetry instance
         if self.llm_loaded and self.llm_loaded.recipe == "llamacpp":
-            return self.llama_telemetry.get_telemetry_data()
+            return self.wrapped_server.telemetry.get_telemetry_data()
 
         # For built-in server, use the existing telemetry
         return {
@@ -1203,6 +1321,34 @@ class Server(ManagementTool):
             ),
         }
 
+    async def get_system_info(self, request: Request):
+        """
+        Return system and device enumeration information.
+        Supports optional 'verbose' query parameter.
+        """
+        from lemonade.common.system_info import (
+            get_system_info_dict,
+            get_device_info_dict,
+            get_system_info as get_system_info_obj,
+        )
+
+        # Get verbose parameter from query string (default to False)
+        verbose = request.query_params.get("verbose", "false").lower() in ["true", "1"]
+
+        info = get_system_info_dict()
+        info["devices"] = get_device_info_dict()
+
+        # Filter out verbose-only information if not in verbose mode
+        if not verbose:
+            essential_keys = ["OS Version", "Processor", "Physical Memory", "devices"]
+            info = {k: v for k, v in info.items() if k in essential_keys}
+        else:
+            # In verbose mode, add Python packages at the end
+            system_info_obj = get_system_info_obj()
+            info["Python Packages"] = system_info_obj.get_python_packages()
+
+        return info
+
     def model_load_failure(self, model_reference: str, message: Optional[str] = None):
         """
         Clean up after a model load failure, then log it and raise
@@ -1212,7 +1358,7 @@ class Server(ManagementTool):
         self.tokenizer = None
         self.model = None
 
-        default_message = f"model {model_reference} not found"
+        default_message = "see stack trace and error message below"
         if message:
             detail = message
         else:
@@ -1237,6 +1383,9 @@ class Server(ManagementTool):
             recipe=config.recipe,
             reasoning=config.reasoning,
             mmproj=config.mmproj,
+            # The pull endpoint will download an upgraded model if available, even
+            # if we already have a local copy of the model
+            do_not_upgrade=False,
         )
 
         # Refresh the list of downloaded models, to ensure it
@@ -1312,9 +1461,9 @@ class Server(ManagementTool):
             ):
                 if (
                     self.llm_loaded.recipe == "llamacpp"
-                    and self.llama_server_process.poll()
+                    and self.wrapped_server.process.poll()
                 ):
-                    # llama-server process has gone away for some reason, so we should
+                    # wrapped server process has gone away for some reason, so we should
                     # proceed with loading to get it back
                     pass
                 else:
@@ -1330,9 +1479,11 @@ class Server(ManagementTool):
             logging.info(f"Loading llm: {config.model_name}")
             try:
                 if config_to_use.recipe == "llamacpp":
-                    self.llama_server_process = llamacpp.server_load(
+                    self.wrapped_server = LlamaServer(self.llamacpp_backend)
+                    self.wrapped_server.load(
                         model_config=config_to_use,
-                        telemetry=self.llama_telemetry,
+                        ctx_size=self.ctx_size,
+                        do_not_upgrade=True,
                     )
 
                 else:
@@ -1372,7 +1523,7 @@ class Server(ManagementTool):
                     await self._generate_semaphore.acquire()
 
             if self.llm_loaded.recipe == "llamacpp":
-                self.llama_server_process.terminate()
+                self.wrapped_server.process.terminate()
 
             self.llm_loaded = None
             self.tokenizer = None
